(define (simple-euler slope-function y0 final-x divisions)
      (next-simple-euler slope-function divisions (/ final-x (+ .0 divisions)) 0. (+ .0 y0) 0))

(define (next-simple-euler slope-function divisions division-length current-x current-y current-division)
  (if (= divisions current-division)
      (cons current-x current-y)
      (next-simple-euler slope-function
			 divisions
			 division-length
			 (flo:+ current-x division-length)
			 (flo:+ current-y (flo:* division-length (slope-function current-x current-y)))
			 (+ current-division 1))))

(define (runge-kutta-2 slope-fn y0 final-x divisions)
  (next-rk2 slope-fn divisions (/ final-x (+ .0 divisions)) 0. (+ y0 .0) 0))

(define (next-rk2 slope-fn divisions step x y div)
  (if (= divisions div)
      (cons x y)
      (let* ((slope-a (slope-fn x y))
	     (slope-b (slope-fn (flo:+ x step)
				(flo:+ y (flo:* step slope-a))))
	     (slope (flo:/ (flo:+ slope-a slope-b) 2.)))
	(next-rk2 slope-fn
		  divisions
		  step
		  (flo:+ x step)
		  (flo:+ y (flo:* step slope))
		  (+ div 1)))))

(define (runge-kutta-4 slope-fn y0 final-x divisions)
  (next-rk4 slope-fn divisions (/ final-x (+ .0 divisions)) (/ (/ final-x (+ divisions .0)) 2) 0. (+ y0 .0) 0))

(define (next-rk4 slope-fn divisions step step2 x y div)
  (if (= divisions div)
      (cons x y)
      (let* ((slope-a (slope-fn x y))
	     (slope-b (slope-fn (flo:+ x step2)
				(flo:+ y (flo:* step2 slope-a))))
	     (slope-c (slope-fn (flo:+ x step2)
				(flo:+ y (flo:* step2 slope-b))))
	     (slope-d (slope-fn (flo:+ x step)
				(flo:+ y (flo:* step slope-c))))
	     (slope (flo:/ (+ slope-a
			      (flo:* 2. slope-b)
			      (flo:* 2. slope-c)
			      slope-d)
			   6.)))
	(next-rk4 slope-fn
		  divisions
		  step
		  step2
		  (flo:+ x step)
		  (flo:+ y (flo:* step slope))
		  (+ div 1)))))

(define (2d-rk4 x-slope-fn y-slope-fn y0 x0 final-t divisions)
  (let* ((current-x (+ x0 .0))
	 (current-y (+ y0 .0))
	 (divisions (+ divisions .0))
	 (division-length (/ final-t divisions))
	 (half-division-length (/ division-length 2)))
    (next-2d-rk4 x-slope-fn 
		 y-slope-fn 
		 divisions 
		 division-length 
		 half-division-length 
		 current-x 
		 current-y 
		 .0)))

(define (next-2d-rk4 x-slope-fn y-slope-fn divisions step step2 x y div)
  (if (= divisions div)
      (cons x y)
      (let* ((slope-a-x (x-slope-fn x y))
	     (slope-a-y (y-slope-fn x y))
	     (x-pt-a (flo:+ x (flo:* step2 slope-a-x)))
	     (y-pt-a (flo:+ y (flo:* step2 slope-a-y)))
	     (slope-b-x (x-slope-fn x-pt-a y-pt-a))
	     (slope-b-y (y-slope-fn x-pt-a y-pt-a))
	     (x-pt-b (flo:+ x (flo:* step2 slope-b-x)))
	     (y-pt-b (flo:+ y (flo:* step2 slope-b-y)))
	     (slope-c-x (x-slope-fn x-pt-b y-pt-b))
	     (slope-c-y (y-slope-fn x-pt-b y-pt-b))
	     (x-pt-c (flo:+ x (flo:* step slope-c-x)))
	     (y-pt-c (flo:+ y (flo:* step slope-c-y)))
	     (slope-d-x (x-slope-fn x-pt-c y-pt-c))
	     (slope-d-y (y-slope-fn x-pt-c y-pt-c))
	     (slope-x (flo:/ (+ slope-a-x
				(flo:* 2. slope-b-x)
				(flo:* 2. slope-c-x)
				slope-d-x)
			     6.))
	     (slope-y (flo:/ (+ slope-a-y
				(flo:* 2. slope-b-y)
				(flo:* 2. slope-c-y)
				slope-d-x)
			     6.)))
	(next-2d-rk4 x-slope-fn
		     y-slope-fn
		     divisions
		     step
		     step2
		     (flo:+ x (flo:* step slope-x))
		     (flo:+ y (flo:* step slope-y))
		     (+ div 1)))))

(display
"
The following procedures are now defined:

runge-kutta-4 - scheme 4th-order Runge-Kutta implementation
runge-kutta-2 - scheme 2nd-order Runge-Kutta (improved Euler) implementation
simple-euler - scheme simple Euler implementation
  arguments: function - function to approximate.  The function 
                        must take two floating-point arguments
                        and return the value of the slope at that point
             y0 - the initial y value of the function
             final-x - the final x-value at which to approximate the 
                       value of y
             divisions - the number of divisions to use.  More accuracy
                         comes with more divisions.
  returns: (x-value . y-value) (a single cons cell, not a list!)

2d-rk4 - scheme 4th-order Runge-Kutta implementation for 2-d
         (parameterized) approximation
  arguments: x-slope-function - the function dx/dt (where t is the parameter)
             y-slope-function - the function dy/dt (where t is the parameter)
             y0 - the initial y value
             x0 - the initial x value
             final-t - the final value of the parameter t
             divisions - the number of divisions to use
  returns: (x-value . y-value)

"
)


